using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace MyApp.Namespace
{
    [Route("[controller]")]
    [ApiController]
    public class ExperienceController : ControllerBase
    {
        private readonly IExperienceService experienceService;
        private readonly IProviderService providerService;

        public ExperienceController(IExperienceService experienceService, IProviderService providerService)
        {
            this.experienceService = experienceService;
            this.providerService = providerService;
        }

        [HttpGet(Name = "ExperiencesGet")]
        [AllowAnonymous] // Alle (også ikke-loggede) må se experiences – svarer til Query 2
        public async Task<IActionResult> Get()
        {
            var result = await experienceService.GetExperiences();
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpGet("{name}", Name = "ExperienceGet")]
        [AllowAnonymous] // Detaljevisning af én experience – tilladt for alle
        public async Task<IActionResult> Get(string name)
        {
            var result = await experienceService.GetExperience(name);
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpGet("search")]
        [AllowAnonymous] // Søgefunktion må bruges uden login
        public async Task<IActionResult> Search(
            [FromQuery] string? name,
            [FromQuery] string? description,
            [FromQuery] int? price)
        {
            var result = await experienceService.SearchExperience(name, description, price);
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpGet("statistics")]
        [Authorize(Roles = "Admin,Manager")] // Statistik (min/avg/max pris) – kun ledelse. Svarer til Query 7
        public async Task<IActionResult> GetStatistics()
        {
            var result = await experienceService.GetStatistics();
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpGet("sales")]
        [Authorize(Roles = "Admin,Manager")] // Antal gæster og omsætning – følsomt, kun for Query 8
        public async Task<IActionResult> GetSales()
        {
            var result = await experienceService.GetSales();
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpPost(Name = "ExperiencePost")]
        [Authorize(Roles = "Admin,Manager")] // Kun autoriserede må oprette experiences
        public async Task<IActionResult> Post([FromBody] CreateExperienceModel experience)
        {
            if (!ModelState.IsValid) return BadRequest(ModelState);

            var result = await experienceService.CreateExperience(experience);
            if (result == null)
            {
                return BadRequest("Could not create experience, provider may be non existing.");
            }

            return CreatedAtRoute("ExperienceGet", new { name = result.Name }, result);
        }

        [HttpPut(Name = "UpdateExperience")]
        [Authorize(Roles = "Admin,Manager")] // Kun ledelsen må opdatere experience-priser
        public async Task<IActionResult> UpdatePrice([FromBody] UpdateExperienceModel experience)
        {
            if (!ModelState.IsValid) return BadRequest(ModelState);

            var result = await experienceService.UpdateExperience(experience);
            if (result == null) return NotFound();
            return Ok(result);
        }

        [HttpDelete("{id}")]
        [Authorize(Roles = "Admin,Manager")] // Kun ledelsen må slette experiences
        public async Task<IActionResult> Delete(int id)
        {
            var result = await experienceService.DeleteExperience(id);
            if (result == null) return NotFound();
            return Ok(result);
        }
    }
}

