using assignment1.Models;
using assignment1.DTO;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using ZiggyCreatures.Caching.Fusion;
using Microsoft.Extensions.Caching.Memory;
using Microsoft.Extensions.Options;
using Serilog;
using DotNetEnv;
using System.Text;
using assignment1.DB;
using assignment1.Interfaces;
using assignment1.Services;
using assignment2.Services;
using assignment2.Models;
using assignment2.Interfaces;
using MongoDB.Bson;

var builder = WebApplication.CreateBuilder(args);

// Mongo + Logging
builder.Services.Configure<LogsDatabaseSettings>(
    builder.Configuration.GetSection("LoggingDB"));

builder.Services.AddDbContext<ExperienceDBContext>();

// Identity + Role support
builder.Services.AddIdentity<ApiUser, IdentityRole>()
    .AddEntityFrameworkStores<ExperienceDBContext>()
    .AddDefaultTokenProviders();

// JWT authentication
builder.Services.AddAuthentication(options =>
{
    options.DefaultAuthenticateScheme = JwtBearerDefaults.AuthenticationScheme;
    options.DefaultChallengeScheme = JwtBearerDefaults.AuthenticationScheme;
})
.AddJwtBearer(options =>
{
    options.TokenValidationParameters = new TokenValidationParameters
    {
        ValidateIssuer = true,
        ValidateAudience = true,
        ValidateLifetime = true,
        ValidateIssuerSigningKey = true,
        ValidIssuer = builder.Configuration["JWT:Issuer"],
        ValidAudience = builder.Configuration["JWT:Audience"],
        IssuerSigningKey = new SymmetricSecurityKey(
            Encoding.UTF8.GetBytes(builder.Configuration["JWT:SigningKey"]))
    };
});

builder.Services.AddAuthorization();

// Controllers & services
builder.Services.AddControllers();
builder.Services.AddTransient<IExperienceService, ExperienceService>();
builder.Services.AddTransient<IProviderService, ProviderService>();
builder.Services.AddTransient<ISharedExperienceService, SharedExperienceService>();
builder.Services.AddTransient<IGuestService, GuestService>();
builder.Services.AddSingleton<ILogService, LogsService>();

// ENV support
Env.Load();

// Caching
builder.Services.AddFusionCache().WithDefaultEntryOptions(new FusionCacheEntryOptions
{
    Duration = TimeSpan.FromMinutes(2),
    Priority = CacheItemPriority.Low
});

// Logging
builder.Services.AddSerilog(options => options.ReadFrom.Configuration(builder.Configuration));

// Swagger + JWT UI support
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(options =>
{
    options.SwaggerDoc("v1", new OpenApiInfo
    {
        Title = "Experience API",
        Version = "v1",
        Description = "An API to manage experiences"
    });

    options.AddSecurityDefinition("Bearer", new OpenApiSecurityScheme
    {
        Name = "Authorization",
        Type = SecuritySchemeType.Http,
        Scheme = "Bearer",
        BearerFormat = "JWT",
        In = ParameterLocation.Header,
        Description = "Indtast din token"
    });

    options.AddSecurityRequirement(new OpenApiSecurityRequirement
    {
        {
            new OpenApiSecurityScheme
            {
                Reference = new OpenApiReference
                {
                    Type = ReferenceType.SecurityScheme,
                    Id = "Bearer"
                }
            },
            Array.Empty<string>()
        }
    });
});

var app = builder.Build();

using (var scope = app.Services.CreateScope())
{
    var context = scope.ServiceProvider.GetRequiredService<ExperienceDBContext>();

    var userManager = scope.ServiceProvider.GetRequiredService<UserManager<ApiUser>>();
    var roleManager = scope.ServiceProvider.GetRequiredService<RoleManager<IdentityRole>>();
    IdentitySeeder.SeedRolesAndUsersAsync(userManager, roleManager).GetAwaiter().GetResult();

    var seeder = new Seeder(context);
    seeder.SeedData();
}

app.UseSwagger();
app.UseSwaggerUI(c =>
{
    c.SwaggerEndpoint("/swagger/v1/swagger.json", "Experience API v1");
    c.RoutePrefix = "";
});

//app.UseSerilogRequestLogging
app.UseMiddleware<LoggerMiddleware>();

app.UseAuthentication();
app.UseAuthorization();

app.MapControllers();

Console.WriteLine("Starting the application on http://localhost:5095");
app.Run();



