
using assignment2.Models;
using Microsoft.EntityFrameworkCore;

public class ExperienceService : IExperienceService
{
    private readonly ExperienceDBContext context;

    public ExperienceService(ExperienceDBContext context)
    {
        this.context = context;
    }
    public async Task<IEnumerable<ExperienceView>> GetExperiences()
    {
        var res = await context.Experiences.ToListAsync();
        return res.ToExperienceView();
    }

    public async Task<ExperienceView> GetExperience(string name)
    {
        var res = await context.Experiences.FirstOrDefaultAsync(e => e.Name == name);
        if (res == null)
        {
            return null;
        }
        return res.ToExperienceView();
    }

    public async Task<Statistics> GetStatistics()
    {
        var experiences = await context.Experiences.ToListAsync();
        if (experiences == null)
        {
            return null;
        }

        var stats = new Statistics
        {
            MaxPrice = experiences.Max(e => e.Price),
            MinPrice = experiences.Min(e => e.Price),
            AveragePrice = experiences.Average(e => e.Price)
        };
        return stats;
    }

    public async Task<IEnumerable<ExperienceSalesView>> GetSales()
    {
        var res = await context.Experiences
            .Include(e => e.Guests)
            .Select(e => new ExperienceSalesView
            {
                Name = e.Name,
                GuestCount = e.Guests.Count,
                TotalPrice = e.Price * e.Guests.Count
            })
            .ToListAsync();
        return res;
    }

    public async Task<ExperienceView> CreateExperience(CreateExperienceModel experience)
    {
        var provider = await context.Providers.FirstOrDefaultAsync(p => p.ProviderId == experience.ProviderId);
        if (provider == null)
        {
            return null;
        }

        var exp = new Experience
        {
            Name = experience.Name,
            Description = experience.Description,
            Price = experience.Price,
            Provider = provider
        };

        context.Experiences.Add(exp);
        await context.SaveChangesAsync();
        return exp.ToExperienceView();
    }

    public async Task<Experience> UpdateExperience(UpdateExperienceModel experienceModel)
    {
        var experience = await context.Experiences.FirstOrDefaultAsync(e => e.ExperienceId == experienceModel.ExperienceId);
        if (experience == null)
        {
            return null;
        }
        experience.Price = experienceModel.Price;
        context.Experiences.Update(experience);
        await context.SaveChangesAsync();
        return experience;
    }

    public async Task<Experience> DeleteExperience(int id)
    {
        var experience = await context.Experiences.FirstOrDefaultAsync(e => e.ExperienceId == id);
        if (experience == null)
        {
            return null;
        }
        context.Experiences.Remove(experience);
        await context.SaveChangesAsync();
        return experience;
    }

    public async Task<IEnumerable<ExperienceView>> SearchExperience(string? name, string? description, int? price)
    {
        var query = context.Experiences.AsQueryable();

        if (!string.IsNullOrEmpty(name))
            query = query.Where(e => e.Name.Contains(name));

        if (!string.IsNullOrEmpty(description))
            query = query.Where(e => e.Description.Contains(description));

        if (price.HasValue)
            query = query.Where(e => e.Price == price.Value);

        var res = await query.ToListAsync();
        return res.ToExperienceView();
    }
}